# https://pytorch.org/hub/pytorch_vision_alexnet/
# https://pytorch.org/vision/main/models/generated/torchvision.models.alexnet.html
# https://discuss.pytorch.org/t/why-does-alexnet-in-torch-vision-use-average-pooling/84313
# https://medium.com/analytics-vidhya/concept-of-alexnet-convolutional-neural-network-6e73b4f9ee30
# https://discuss.pytorch.org/t/alexnet-input-size-224-or-227/41272/3
# https://androidkt.com/calculating-number-parameters-pytorch-model/

import numpy as np
import cv2 as cv
from torchvision import transforms
import torchvision.models as models
import torch

preprocess = transforms.Compose([
    transforms.ToTensor(),
    transforms.Resize(256),
    transforms.CenterCrop(224),   
    transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]),
])


model_alexnet = models.alexnet(weights="AlexNet_Weights.IMAGENET1K_V1").eval()
if torch.cuda.is_available():
    model_alexnet = model_alexnet.cuda()

print(f"model_alexnet: {model_alexnet}")
    
from torchsummary import summary
print(f"summary: {summary(model_alexnet, (3, 224, 224))}")

#FREEZE ALL
for p in model_alexnet.parameters():
    p.requires_grad = False        

print([n for n, _ in model_alexnet.named_children()])
print(f"model_alexnet.classifier: {model_alexnet.classifier}")
print(f"model_alexnet.classifier: {model_alexnet.classifier[-1]}")

# nn.Linear have requires_grad=True by default
model_alexnet.classifier[-1] = torch.nn.Linear(in_features=4096, out_features=2)

print(f"model_alexnet: {model_alexnet}")

# optim.SGD(model_alexnet.fc.parameters())

